using System;
using System.IO;
using System.Security.Cryptography;
using System.Text;
using System.Windows.Forms;

namespace P2PBackupClient
{
	[Serializable]
	public class BFile
	{
		private string fileName;	// the original file name
		private string originalFileName; // the whole original path to the file
		private string substituteFileName; // the hashed filename
		private long fileSize;
		private string pathTemp;	// the path to the temporary directory
		private string clientName;	// the username of the destination client
		private bool sent = false;
		private bool fetched = false;
		private bool indexFile = false;
		
		public string OriginalFileName
		{
			set{originalFileName = value;}
			get{return originalFileName;}
		}

		public string SubstituteFileName
		{
			set{substituteFileName = value;}
			get{return substituteFileName;}
		}

		public string FileName
		{
			set{fileName = value;}
			get{return fileName;}
		}

		public string PathTemp
		{
			set{pathTemp = value;}
			get{return pathTemp;}
		}

		public long FileSize
		{
			set{fileSize = value;}
			get{return fileSize;}
		}

		public string ClientName
		{
			get {return clientName; }
			set {clientName = value; }
		}

		public bool Sent
		{
			set{sent = value;}
			get{return sent;}
		}

		public bool Fetched
		{
			set{fetched = value;}
			get{return fetched;}
		}

		public bool IndexFile
		{
			set{indexFile = value;}
			get{return indexFile;}
		}

		public BFile()
		{
		
		}

		public BFile(string fN)
		{
			fileName = fN;
		}

		public BFile(FileInfo fi, string pT)
		{
			fileName = fi.Name;
			originalFileName = fi.FullName;
			DateTime time = DateTime.Now;
			substituteFileName = HashString(fi.FullName + time.ToShortDateString()+ time.ToShortTimeString());
			pathTemp = pT;
			//fileSize = 1000;//Testakod
		}

		/// <summary>
		/// Hashes a string
		/// </summary>
		/// <param name="data">string to hash</param>
		/// <returns>hashed string</returns>
		private string HashString(string data)
		{
			byte[] toHash = Encoding.ASCII.GetBytes(data);
			byte [] hashedResult;

			SHA1Managed hashAlg = new SHA1Managed();
			hashedResult = hashAlg.ComputeHash(toHash);
			return ByteArrayToString(hashedResult);
		}

		/// <summary>
		/// Takes a bytearray and creates a string.
		/// </summary>
		/// <param name="toConvert">byte[] to convert</param>
		/// <returns>string representation of byte[]</returns>
		private string ByteArrayToString(byte [] toConvert)
		{
			StringBuilder sb = new StringBuilder(toConvert.Length);
			for (int i = 0; i < toConvert.Length - 1; i++)
			{
				sb.Append(toConvert[i].ToString("X"));
			}
			return sb.ToString();
		}

		/// <summary>
		/// Encrypts a file
		/// </summary>
		/// <param name="pubKey">the user's public key used to encrypt the session key</param>
		public void EncryptFile(RSACryptoServiceProvider keyPair)
		{	
			FileStream fsCrypt = null;
			FileStream fsKey = null;
			FileStream fsInput = null;
			CryptoStream cs = null;

			byte [] key;
			byte [] IV;
			
			try
			{
				UnicodeEncoding UE = new UnicodeEncoding();

				string fullpath = pathTemp + "\\" + substituteFileName;

				fsKey = new FileStream(fullpath, FileMode.Create);

				// generate the session key
				RijndaelManaged AES = new RijndaelManaged();
				AES.GenerateIV();
				AES.GenerateKey();

				key = AES.Key;
				IV = AES.IV;

				using (fsKey)
				{
					// encrypt iv and key with the public key
					byte [] encryptedIV = keyPair.Encrypt(AES.IV, false);
					byte [] encryptedKey = keyPair.Encrypt(AES.Key, false);

					// write the encrypted iv and key
					fsKey.Write(encryptedIV, 0, encryptedIV.Length);
					fsKey.Flush();
					fsKey.Write(encryptedKey, 0, encryptedKey.Length);
					fsKey.Flush();
				}

				fsCrypt = new FileStream(fullpath, FileMode.Append);
				ICryptoTransform aesencrypt = AES.CreateEncryptor(key, IV);
				cs = new CryptoStream(fsCrypt, aesencrypt, CryptoStreamMode.Write);

				fsInput = new FileStream(originalFileName, FileMode.Open);

				int data;

				// write all data to the crypto stream and flush it
				while ((data = fsInput.ReadByte()) != -1)
					cs.WriteByte((byte)data);

				cs.FlushFinalBlock();

				FileInfo fInfo = new FileInfo(fullpath);
				fileSize = fInfo.Length;
			}
			catch (Exception ex)
			{
				MessageBox.Show(ex.Message,"BFile.EncryptFile");
			}
			finally
			{	
				fsInput.Close();
				cs.Close();
				fsCrypt.Close();
			}
		}

		/// <summary>
		/// Decrypts a file
		/// </summary>
		/// <param name="keyPair">the user's public key used to encrypt the session key</param>
		public void DecryptFile(RSACryptoServiceProvider keyPair)
		{
			FileStream fsKey = null;
			FileStream fsOutput = null;
			CryptoStream cs = null;

			byte [] key;
			byte [] IV;

			// decrypt iv and key with the public key
			byte [] encryptedIV = new byte[256];
			byte [] encryptedKey = new byte[256];

			try
			{
				UnicodeEncoding UE = new UnicodeEncoding();

				// path to the encrypted files
				string fullpath = "Temp" + "\\" + substituteFileName;

				fsKey = new FileStream(fullpath, FileMode.Open);

				// read the session key from the file
				fsKey.Read(encryptedIV, 0, 256);
				fsKey.Read(encryptedKey, 0, 256);

				// decrypt the session key
				IV = keyPair.Decrypt(encryptedIV, false);
				key = keyPair.Decrypt(encryptedKey, false);

				// kan man typ skriva
				// AES.Key = keyPair.Decrypt(encryptedKey, false);
				RijndaelManaged AES = new RijndaelManaged();
				AES.Key = key;
				AES.IV = IV;

				ICryptoTransform aesencrypt = AES.CreateDecryptor();
				cs = new CryptoStream(fsKey, aesencrypt, CryptoStreamMode.Read);

				// get the directory name
				string directoryName = Path.GetDirectoryName(originalFileName);
				// create the directory
				Directory.CreateDirectory(directoryName);
				//DirectoryInfo di = Directory.CreateDirectory(directoryName);

				fsOutput = new FileStream(originalFileName, FileMode.Create);

				int data;

				// read the encrypted file and decrypt it
				while ((data = cs.ReadByte()) != -1)
					fsOutput.WriteByte((byte)data);
			}
			catch (Exception ex)
			{
				MessageBox.Show(ex.Message,"BFile.DecryptFile");
			}
			finally
			{
				fsOutput.Close();
				cs.Close();
			}
		}
	}
}
