using System;
using System.Collections;
using System.IO;
using System.Security.Cryptography;
using System.Text;
using System.Windows.Forms;

namespace P2PBackupClient
{
	/// <summary>
	/// Summary description for BFileList.
	/// </summary>
	public class BFileList:ArrayList
	{
		private BFile bf;
		private string backupName;
		private bool saveBackupLocal = true;
		private FileHandle fileHandle = new FileHandle();
		private string indexFileName = "";
		public delegate void UpdateGUIHandler (string command, string param);
	
		public BFileList()
		{
			
		}

		public string BackupName
		{
			get {return backupName; }
			set {backupName = value; }
		}

		public bool SaveBackupLocal
		{
			get {return saveBackupLocal; }
			set {saveBackupLocal = value; }
		}

		public string IndexFileName
		{
			get {return indexFileName; }
			set {indexFileName = value; }	
		}

		/// <summary>
		/// Get all files in directory and subdirectories.
		/// Creates a BFile for each file in which info about file is stored.
		/// Adds every BFile to BFileList(this)
		/// </summary>
		/// <param name="dInf">directory to backup</param>
		public void GetFileInfo(DirectoryInfo dInf, string pathTemp)
		{
			FileInfo[] filesInFolder = new FileInfo[dInf.GetFiles().Length];
			filesInFolder = dInf.GetFiles();
			int i = 0;
			while(i < filesInFolder.Length)
			{
				bf = new BFile(filesInFolder[i], pathTemp);
				this.Add(bf);
				i++;
			}
			i = 0;
			DirectoryInfo[] dInf2 = dInf.GetDirectories();
			while(i < dInf2.Length)
			{
				GetFileInfo(dInf2[i], pathTemp);
				i++;
			}
		}

		/// <summary>
		/// Get total size for files to backup.
		/// </summary>
		/// <returns>total backup size as string</returns>
		public string GetBackupSize()
		{
			long bSize = 0;
			foreach (BFile bf in this)
			{
				bSize = bSize + bf.FileSize;
			}
			return bSize.ToString();
		}
		
		/// <summary>
		/// Loops through all files in the BFileList and forwards them to Encryption
		/// Sets the filesize on return
		/// </summary>
		/// <param name="nB">the name of the backup</param>
		/// <param name="kP">the user's public key used to encrypt the session key</param>
		public void GetFileToEncrypt(string nB, RSACryptoServiceProvider kP)
		{
			backupName = nB;

			foreach (BFile bf in this)
			{
				bf.EncryptFile(kP);
			}
		}

		/// <summary>
		/// Loops through all files in the BFileList and forwards them to Decryption
		/// </summary>
		/// <param name="kP">the user's public key used to encrypt the session key</param>
		public void GetFileToDecrypt(RSACryptoServiceProvider kP)
		{
			foreach (BFile bf in this)
			{
				bf.DecryptFile(kP);
			}
		}

		/// <summary>
		/// Deletes all the encrypted files(local copy) after backup
		/// </summary>
		public void DeleteEncryptedFiles()
		{
			foreach (BFile bf in this)
			{
				try
				{
					File.Delete(bf.PathTemp + "\\" + bf.SubstituteFileName);
				}
				catch(Exception ex)
				{
					MessageBox.Show(ex.Message,"BFileList.DeleteEncryptedFiles"); 
				}
			}
		}
	}
}
