using System;
using System.IO;
using System.Security.Cryptography;
using System.Windows.Forms;
using System.Text;
using System.Runtime.Serialization.Formatters.Binary;
using System.Runtime.Serialization;

namespace P2PBackupClient
{
	/// <summary>
	/// Summary description for FileHandle.
	/// </summary>
	public class FileHandle
	{
		private StreamWriter fileWriter;
		private StreamReader fileReader;
		private FileStream keyFile;
		
		public FileHandle()
		{
			//
			// TODO: Add constructor logic here
			//
		}

		/// <summary>
		/// Saves the size and path of the share to file
		/// </summary>
		/// <param name="size">share size</param>
		/// <param name="path">share path</param>
		public void SaveShareSize(string size, string path)
		{
			long lSize = Int64.Parse(size);
			lSize = lSize * 1048576;
			lSize.ToString();
			try
			{
				fileWriter = new StreamWriter("share.cfg", false, Encoding.Unicode);
				fileWriter.Write(size + " " + path);
			}
			catch(Exception ex)
			{
				MessageBox.Show(ex.Message,"FileHandle.SaveShareSize"); 
			}
			finally
			{
				fileWriter.Close();
			}
		}

		/// <summary>
		/// Gets the share path and share size from file.
		/// </summary>
		/// <returns>pathsize[0:size]pathsize[1:path]</returns>
		public string[] GetSharePathSize()
		{
			string [] sizepath = new string[2];
		
			string read = "";
			if (File.Exists("share.cfg"))
			{
				try
				{
					fileReader = new StreamReader("share.cfg", Encoding.Unicode);
					read = fileReader.ReadLine();
				}
				catch(Exception ex)
				{
					MessageBox.Show(ex.Message,"FileHandle.GetSharePathSize"); 
				}
				finally
				{
					fileReader.Close();
				}
				char[] separator = {' '};
				sizepath = read.Split(separator,2);
			}
			return sizepath;
		}

		/// <summary>
		/// Saves the asymmetric key pair to file
		/// </summary>
		/// <param name="keyByte">byte[] representation of key pair</param>
		public void SaveKeyPair(byte[] keyByte)
		{
			// Create the key path string.
			string keyPath;    // The location of the keypair.
			keyPath = Application.ExecutablePath;
			keyPath = keyPath.Substring(0, keyPath.LastIndexOf(@"\") + 1) + "keypair";
			if (!File.Exists(keyPath))
			{
				try
				{
					// Save the keypair to a file.
					keyFile = File.Open(keyPath, FileMode.CreateNew);
					keyFile.Write(keyByte, 0, keyByte.Length);
				}
				catch(Exception ex)
				{
					MessageBox.Show(ex.Message,"FileHandle.SaveKeyPair"); 
				}
				finally
				{
					keyFile.Close();
				}
			}	
		}

		/// <summary>
		/// Reads the keypair from file
		/// </summary>
		/// <returns>the keypair</returns>
		public Byte[] GetKeyPair()
		{
			byte[] keyByte = null;    // Converted key data.
			string keyPath;    // The location of the keypair.
			keyPath = Application.ExecutablePath;
			keyPath = keyPath.Substring(0, keyPath.LastIndexOf(@"\") + 1) + "keypair";
			if (File.Exists(keyPath))
			{
				try
				{
					// Open the key file for reading.
					keyFile = File.Open(keyPath, FileMode.Open);
					keyByte = new Byte[keyFile.Length];
					keyFile.Read(keyByte, 0, (Int32)keyFile.Length);
				}
				catch(Exception ex)
				{
					MessageBox.Show(ex.Message,"FileHandle.GetKeyPair"); 
				}
				finally
				{
					keyFile.Close();
				}
			}
			return keyByte;
		}
		
		/// <summary>
		/// Creates the indexfile for a backup.
		/// </summary>
		/// <param name="nameBackup">name for backup</param>
		public void CreateBackupIndex(string nameBackup)
		{
			try
			{
				fileWriter = new StreamWriter(nameBackup + ".p2p", false);
				//fileWriter.Write("");
			}
			catch(Exception ex)
			{
				MessageBox.Show(ex.Message,"FileHandle.CreateBackupIndex"); 
			}
			finally
			{
				fileWriter.Close();
			}
		}

		/// <summary>
		/// writes data about the file being backupped to the index file
		/// </summary>
		/// <param name="bf">the file being backupped, serialized</param>
		/// <param name="nameBackup">the name of the backup</param>
		public void SaveToIndexFile(BFile bf, string nameBackup)
		{
			fileWriter = new StreamWriter(nameBackup + ".p2p", true);
			BinaryFormatter formatter = new BinaryFormatter();
			
			try
			{
				using (fileWriter)
				{
					formatter.Serialize(fileWriter.BaseStream, bf);
				}
			}
			catch (Exception ex)
			{
				MessageBox.Show(ex.Message,"FileHandle.SaveToIndexFile");
			}
		}

		/// <summary>
		/// Gets all BFile from indexfile and returns them in a BFileList
		/// </summary>
		/// <param name="nameBackup">name for backup</param>
		/// <returns>BFileList with all BFile:s</returns>
		public BFileList GetFromIndexFile(string nameBackup)
		{
			BFileList bfl = new BFileList();
			BFile bf = new BFile();
			
			BinaryFormatter formatter = new BinaryFormatter();
			
			try
			{
				fileReader = new StreamReader(nameBackup + ".p2p");
				using (fileReader)
				{
					while(fileReader.BaseStream.Position < fileReader.BaseStream.Length)
					{
					
						bf = (BFile)formatter.Deserialize(fileReader.BaseStream);
						bfl.Add(bf);
					}					
				}
				return bfl;
			}
			catch (Exception ex)
			{
				MessageBox.Show(ex.Message,"FileHandle.GetFromIndexFile");
			}
			return null;
		}

		/// <summary>
		/// Deletes a index-file
		/// </summary>
		/// <param name="nameBackup">name of backup</param>
		public void DeleteIndexFile(string nameBackup)
		{
			try
			{
				File.Delete(nameBackup + ".p2p");
			}
			catch (Exception ex)
			{
				MessageBox.Show(ex.Message,"FileHandle.DeleteIndexFile");
			}
		}

	}
}
