using System;
using System.Net;
using System.Net.Sockets;
using System.IO;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using System.Security.Cryptography;

namespace P2PBackupClient
{
	/// <summary>
	/// Summary description for Session.
	/// </summary>
	public class Session
	{
		private Socket client;
		private string clientName;
		private string clientKey;
		private string clientIp;
		private StreamWriter clientWriter;
		private StreamReader clientReader;
		private bool run;
		private RSACryptoServiceProvider myKeyPairCrypto;
		private User user;
		private BFile bf;
		private string sharePath;
		private string recoveryPath = "Temp";
		private NetworkStream stream;
		private Verification ver;
		private int sessionType;//1:SendBack, 2:GetRecov, 3:ReceiveBack, 4:SendRecov}

		public delegate void RemoveSessionHandler (Session s, int SessionType);
		public event RemoveSessionHandler RemoveSessionEvent;//User.RemoveSession()
		public delegate void FileSentHandler (bool sent, BFile bFSent);
		public event FileSentHandler FileSentEvent;//User.FileSent()
		public delegate void FileReceivedHandler (bool received, BFile bFReceived);
		public event FileReceivedHandler FileReceivedEvent;//User.
		


		public Socket Client
		{
			set { client = value; }
		}

		public string ClientIp
		{
			get { return clientIp; }
		}

		public string SharePath
		{
			set { sharePath = value; }
		}

		public int SessionType
		{
			set { sessionType = value; }
		}

		public BFile BF
		{
			set { bf = value; }
		}

		public Session(int sT, User u, string cIp, string cN, string cK, RSACryptoServiceProvider kpc)
		{
			sessionType = sT;
			user = u;
			clientIp = cIp;
			clientName = cN;
			clientKey = cK;
			myKeyPairCrypto = kpc;
			ver = new Verification(myKeyPairCrypto,user.Username,clientKey);
		}

		public Session(int sT, User u, string cIp, string cN, string cK, RSACryptoServiceProvider kpc,BFile bF)
		{
			sessionType = sT;
			user = u;
			clientIp = cIp;
			clientName = cN;
			clientKey = cK;
			myKeyPairCrypto = kpc;
			bf = bF;
			ver = new Verification(myKeyPairCrypto,user.Username,clientKey);	
		}

		public Session(int sT, User u, string cIp, string cN, string cK, RSACryptoServiceProvider kpc, string sP)
		{
			sessionType = sT;
			user = u;
			clientIp = cIp;
			clientName = cN;
			clientKey = cK;
			myKeyPairCrypto = kpc;
			sharePath = sP;
			ver = new Verification(myKeyPairCrypto,user.Username,clientKey);
		}

		/// <summary>
		/// Connectes to client2
		/// </summary>
		public void ConnectToClient()
		{
			try
			{
				client = new Socket(AddressFamily.InterNetwork,SocketType.Stream,ProtocolType.Tcp);
				IPAddress addr = IPAddress.Parse(clientIp);
				IPEndPoint dest = new IPEndPoint(addr,52562);
				client.Connect(dest);
				if (client.Connected == true)
				{
					StartListening();
					SendDigitalSignature("DS1");
				}
			}
			catch(SocketException)
			{
				MessageBox.Show("Det gick inte att ansluta!");
			}
			catch(Exception ex)
			{
				MessageBox.Show(ex.Message,"Session.ConnectToClient"); 
			}
		}

		
		/// <summary>
		/// Sets the streams to the socket, creates a new thread that listens
		/// </summary>
		public void StartListening ()
		{
			try
			{
				stream = new NetworkStream(client);
				clientWriter = new StreamWriter(stream, Encoding.Unicode);
				clientReader = new StreamReader(stream, Encoding.Unicode);
				Thread listening = new Thread(new ThreadStart(Listen));		
				listening.IsBackground = true;
				listening.Start();
				run = true;
			}
			catch(Exception ex)
			{
				MessageBox.Show(ex.Message,"Session.StartListening"); 
			}
		}
		
		/// <summary>
		/// Listens to the connected client and forwards the messages
		/// </summary>
		private void Listen()
		{
			try
			{
				using(clientReader)
				{
					while(run)
					{ 
						Decode(clientReader.ReadLine()); 
					}
				}
			}
				// det r hit man kommer nr klienten stngs
			catch(IOException)
			{
				this.Disconnect();
			}
			catch(Exception ex)
			{
				MessageBox.Show(ex.Message,"Session.Listen"); 
			}
		}

		/// <summary>
		/// Decodes the incoming messages
		/// </summary>
		/// <param name="message">the incoming message</param>
		private void Decode(string message)
		{
			try
			{
				if (message == null)
					return;
				string type="";
				char[] separator = {' '};
				string[] decoded = message.Split(separator);
				if(decoded[0].Length==3)
					type = decoded[0];
				switch(type)
				{
						//BOKSTAVSORDNING!!!
					case "CRC"://Checksum
						if(decoded.Length == 2)
						{
						}
						break;
					case "DS1"://Digital Signature from klient1
						if(decoded.Length == 3)
						{
							ver.CheckDigitalSignature(decoded[1],decoded[2]);
							if(ver.ClientVerification)
							{
								SendVerOK("204");
								SendDigitalSignature("DS2");
							}
						}
						break;
					case "DS2"://Digital Signature from klient2
						if(decoded.Length == 3)
						{
							ver.CheckDigitalSignature(decoded[1],decoded[2]);
							if((ver.ClientVerification)&& (ver.MyVerification))
							{
								SendVerOK("213");
								ContinueWork();
							}
						}
						break;
					case "FIL":
						if(decoded.Length == 3)
							if(sessionType == 2)
								ReceiveFile(this.recoveryPath + "\\" + decoded[1], decoded[2]);	
							if(sessionType == 3)
								ReceiveFile(this.sharePath + "\\" + decoded[1], decoded[2]);
						break;
					case "GET":
						if(decoded.Length == 2)
						{
							if(sessionType == 4)
							{
								if(File.Exists(this.sharePath + "\\" + decoded[1]))
								{
									CheckForFileSize(decoded[1]);
								}
								else{}//om fil inte finns
							}
						}
						break;

						//NUMMERORDNING !!!
					case "204"://Verification of client1 succeded
						if(decoded.Length == 1)
						{
							ver.MyVerification = true;
						}
						break;
					case "205": //File received
						if(decoded.Length == 1 && RemoveSessionEvent != null)
						{
							if(sessionType == 1 && FileSentEvent != null)
							{
								Disconnect();
								bf.ClientName = this.clientName;
								RemoveSessionEvent(this,sessionType);
								FileSentEvent(true,this.bf);
							}
							if(sessionType == 4)
							{
								Disconnect();
								RemoveSessionEvent(this,sessionType);
							}

						}
						break;
					case "211": //Ok to send file
						if(decoded.Length == 1)
						{
							if(sessionType == 1)
								TransferFile(bf.PathTemp + "\\" + bf.SubstituteFileName);
							if(sessionType == 4)
								TransferFile(this.sharePath + "\\" + bf.FileName);
						}
						break;
					case "213"://Verification of client2 succeded
						if(decoded.Length == 1)
						{
							ver.MyVerification = true;
							ContinueWork();
						}
						break;
				}
			}
			catch(Exception ex)
			{
				MessageBox.Show(ex.Message,"Session.Decode"); 
			}
		}

		/// <summary>
		/// Closes the socket and streams for this session
		/// if the connected client has disconnected
		/// Sets the bool attributes to false
		/// </summary>
		private void Disconnect()
		{
			try
			{
				run = false;
				if (clientWriter != null)
					clientWriter.Close ();
				if (clientReader != null)
					clientReader.Close ();

				Thread.Sleep(1024);
				if(this.client !=null && this.client.Connected)
				{
					this.client.Shutdown(SocketShutdown.Both);
					this.client.Close();
				}
			}
			catch(Exception ex)
			{
				MessageBox.Show(ex.Message,"Session.Disconnect"); 
			}
		}


		/// <summary>
		/// Sends original message and digital signature from client1 with command DS1.
		/// Sends original message and digital signature from client2 with command "DS2"
		/// </summary>
		/// <param name="command">DS1 or DS2</param>
		public void SendDigitalSignature(string command)
		{
			string original = "";
			string digSig = "";
			digSig = ver.CreateDigitalSignature(out original);
			try
			{
				clientWriter.WriteLine(command + " " + original + " " + digSig);
				clientWriter.Flush();
			}
			catch(IOException)
			{
				//this.Disconnect(false);
			}
			catch(Exception ex)
			{
				MessageBox.Show(ex.Message,"Session.SendDigitalSignature"); 
			}
		}

		/// <summary>
		/// Sends a to other client that verification was okay
		/// </summary>
		/// <param name="command">command</param>
		public void SendVerOK(string command)
		{
			try
			{
				clientWriter.WriteLine(command);
				clientWriter.Flush();
			}
			catch(IOException)
			{
				//this.Disconnect(false);
			}
			catch(Exception ex)
			{
				MessageBox.Show(ex.Message,"Session.SendVerOK"); 
			}
		}

		/// <summary>
		/// Continues work after verification. What to do depends on sessionType. 
		/// </summary>
		private void ContinueWork()
		{//1:SendBack, 2:GetRecov, 3:ReceiveBack, 4:SendRecov}
			if (sessionType == 1)
			{
				bf.ClientName = this.clientName;
				SendFile(bf.SubstituteFileName,bf.FileSize.ToString());
			}
			else if (sessionType == 2)
			{
				GetFile(bf.SubstituteFileName);
			}
		}

		/// <summary>
		/// This is used when client sends recovery to other client, to find out the filesize. 
		/// </summary>
		/// <param name="fileName">filename of file to send</param>
		private void CheckForFileSize(string fileName)
		{
			try
			{
				DirectoryInfo dInf = new DirectoryInfo(this.sharePath + "\\");
				FileInfo[] fi = dInf.GetFiles(fileName);
				bf = new BFile(fileName);
				SendFile(fileName,fi[0].Length.ToString());
			}
			catch(Exception ex)
			{
				MessageBox.Show(ex.Message,"Session.CheckForFileSize"); 
			}

		}


		/// <summary>
		/// Sends filname and filesize to client2
		/// </summary>
		/// <param name="fileName">filename</param>
		/// <param name="fileSize">filesize</param>
		private void SendFile(string fileName, string fileSize)
		{
			try
			{
				clientWriter.WriteLine("FIL " + fileName + " " + fileSize);
				clientWriter.Flush();
			}
			catch(IOException)
			{
				//this.Disconnect(false);
			}
			catch(Exception ex)
			{
				MessageBox.Show(ex.Message,"Session.SendFile"); 
			}
		}

		/// <summary>
		/// Tells the other client that a recoveryfile should be fetched
		/// </summary>
		/// <param name="fileName"></param>
		private void GetFile(string fileName)
		{
			try
			{
				clientWriter.WriteLine("GET " + fileName);
				clientWriter.Flush();
			}
			catch(IOException)
			{
				//this.Disconnect(false);
			}
			catch(Exception ex)
			{
				MessageBox.Show(ex.Message,"Session.GetFile"); 
			}
		}

		/// <summary>
		/// Transfers file to client2.
		/// </summary>
		/// <param name="path">path to file, including filename</param>
		private void TransferFile(string path)
		{
			//NetworkStream fileNet = new NetworkStream(client);
			FileStream file = null;
			try
			{
				long rdbyte = 0 ;
				byte[] buffed = new byte[1024] ;
				file = new FileStream(path,FileMode.Open,FileAccess.Read);
				while(rdbyte<file.Length && stream.CanWrite)
				{
					int i = file.Read(buffed,0,buffed.Length) ;
					stream.Write(buffed,0,i);
					stream.Flush();
					rdbyte=rdbyte+i ;
				}
			}
			catch(IOException)
			{
				Disconnect();
			}
			catch(Exception ex)
			{
				MessageBox.Show(ex.Message,"Session.TransferFile"); 
			}
			finally
			{
				file.Close();
				//fileNet.Close();
			}	
		}

		/// <summary>
		/// Receives files from other client.
		/// Tells other client that file has been received.
		/// </summary>
		/// <param name="path">path to where file should be stored, including filename</param>
		/// <param name="fileSize">filesize</param>
		private void ReceiveFile(string path, string fileSize)
		{
			
			FileStream file = null;;
			try
			{
				long rdbyte = 0;
				byte[] buffer = new byte[1024] ;	
				file = new FileStream(path, FileMode.OpenOrCreate , FileAccess.Write);

				using (file)
				{
					clientWriter.WriteLine("211");
					clientWriter.Flush();

					while(rdbyte <Int64.Parse(fileSize) && stream.CanRead)
					{//hmtar fil
						int i = stream.Read(buffer,0,buffer.Length) ;
						file.Write(buffer,0,i) ;
						rdbyte = rdbyte+i;
					}
				}
				
				clientWriter.WriteLine("205");
				clientWriter.Flush();
				Disconnect();
				RemoveSessionEvent(this,sessionType);
				if(sessionType == 2 && FileReceivedEvent != null)
				{
					FileReceivedEvent(true,bf);
					
				}
			}
			catch(IOException)
			{
				Disconnect();
			}
			catch(Exception ex)
			{
				MessageBox.Show(ex.Message,"Session.ReceiveFile"); 
			}
		}		
	}
}
