using System;

using System.Text;

using System.Security.Cryptography;
using System.Windows.Forms;

namespace P2PBackupClient
{
	/// <summary>
	/// Summary description for Verification.
	/// </summary>
	public class Verification
	{
		private RSACryptoServiceProvider myKeyPairCrypto;
		private string myUserName;
		private string clientKey;
		private bool myVerification = false;
		private bool clientVerification = false;

		public Verification(RSACryptoServiceProvider mKP, string uN, string cK)
		{
			myKeyPairCrypto= mKP;
			myUserName = uN;
			clientKey = cK;
		}

		public bool MyVerification
		{
			get { return myVerification; }
			set { myVerification = value; }
		}

		public bool ClientVerification
		{
			get { return clientVerification; }
		}

		/// <summary>
		/// Creates digital signature with users private key. 
		/// Original consists of username,timestamp and random number.
		/// </summary>
		/// <param name="original">the original message</param>
		/// <returns>the digital signature in Base64String</returns>
		public string CreateDigitalSignature(out string original)
		{
			string timeString;
			string trimmedTime;
			DateTime time = DateTime.Now;
			Random rand = new Random((int)time.Ticks);
			int no;
			no = rand.Next();//random number
			timeString = time.ToString();
			trimmedTime = timeString.Remove(10, 1);//timestamp
			original =  myUserName + trimmedTime + no;//adds username with rand number and timestamp
			
			byte[] hashedData = HashString(original);
			byte[] signature = myKeyPairCrypto.SignHash(hashedData,CryptoConfig.MapNameToOID("SHA1"));//sign with users privatekey
			
			return Convert.ToBase64String(signature);
		}

		/// <summary>
		/// Checks the digital signature for client1 and client2
		/// </summary>
		/// <param name="command">DS1 or DS2</param>
		/// <param name="original">original message</param>
		/// <param name="digSig">digital signature as string</param>
		public void CheckDigitalSignature(string original, string digSig)
		{
			byte[] hashedData = HashString(original);//hashes originalmessage
			byte [] signature = Convert.FromBase64String(digSig);

			RSACryptoServiceProvider RSACrypto= new RSACryptoServiceProvider();
			RSACrypto.FromXmlString(clientKey);
			
			clientVerification = RSACrypto.VerifyHash(hashedData,CryptoConfig.MapNameToOID("SHA1"), signature);//verifies the digital signature with client2 public key
		}

		/// <summary>
		/// Converts string to byte and hash
		/// </summary>
		/// <param name="toHash">string to hash</param>
		/// <returns>hashed value in bytearray</returns>
		private byte[] HashString (string toHash)
		{
			byte [] hashedData;
			byte[] byteToHash = Encoding.Unicode.GetBytes(toHash);
			SHA1Managed hashAlg = new SHA1Managed();
			hashedData = hashAlg.ComputeHash(byteToHash);
			return hashedData;	
		}
	}
}
