using System;
using System.Data.OleDb;
using System.Data;
using System.Windows.Forms;

namespace P2PBackupHub
{
	/// <summary>
	/// Handles all communication with the database
	/// </summary>
	public class DBHandle
	{
		private string connStr = "Provider = Microsoft.Jet.OLEDB.4.0;Data Source = P2PBackup.mdb";

		public DBHandle()
		{

		}

		/// <summary>
		/// Adds a new user to the database
		/// </summary>
		/// <param name="uName">username</param>
		/// <param name="pass">password</param>
		public void AddUser(string uName, string pass)
		{
			string query = "INSERT INTO Users (userName, pass) VALUES ('" + uName + "', '" + pass + "')";
			OleDbConnection dbConnection = new OleDbConnection(connStr);
			OleDbCommand dbCommand = new OleDbCommand(query);

			try
			{
					dbCommand.Connection = dbConnection;
					dbConnection.Open();
					dbCommand.ExecuteNonQuery();
			}	
			catch(Exception ex)
			{
				MessageBox.Show(ex.Message, "Hub.DBHandle.AddUser");
			}
			finally
			{
					dbCommand.Connection.Close();
			}
		}

		/// <summary>
		/// Deletes a user
		/// </summary>
		/// <param name="uN">the username to delete</param>
		public void DeleteUser(string uN)
		{
			string query = "DELETE FROM Users WHERE userName = '" + uN + "'" ;
			OleDbConnection dbConnection = new OleDbConnection(connStr);
			OleDbCommand dbCommand = new OleDbCommand(query);

			try
			{
				dbCommand.Connection = dbConnection;
				dbConnection.Open();
				dbCommand.ExecuteNonQuery();
			}	
			catch(Exception ex)
			{
				MessageBox.Show(ex.Message, "Hub.DBHandler.DeleteUser");
			}
			finally
			{
				dbCommand.Connection.Close();
			}	
		}

		/// <summary>
		/// Checks if the connecting user with this username exists in the database
		/// Gets all the data about the existing user in the database 
		/// </summary>
		/// <param name="uName">the user's username</param>
		/// <returns>a User if the connecting user exists, null otherwise</returns>
		public User UserExists(string uName)
		{
			string query = "SELECT * FROM Users WHERE Users.userName = '" + uName + "'";
			DataSet ds = new DataSet();
			OleDbConnection dbConnection = new OleDbConnection(connStr);
			bool locked;

			try
			{
				OleDbDataAdapter dataAdapter = new OleDbDataAdapter(query, connStr);
				dataAdapter.Fill (ds, "Users");
			}
			catch(Exception ex)
			{
				MessageBox.Show(ex.Message, "Hub.DBHandle.UserExists");
			}
			finally
			{
				dbConnection.Close ();
			}

			// check if there's anything in the dataset
			int count = ds.Tables["Users"].Rows.Count;
			if (count > 0)
			{
				int uid = Int32.Parse(ds.Tables["Users"].Rows[0][0].ToString());
				string name =  ds.Tables["Users"].Rows[0][1].ToString();
				string pass = ds.Tables["Users"].Rows[0][2].ToString();
				string key = ds.Tables["Users"].Rows[0][4].ToString();
				long share = Int64.Parse(ds.Tables["Users"].Rows[0][5].ToString());
				long access = Int64.Parse(ds.Tables["Users"].Rows[0][6].ToString());
				long avail = Int64.Parse(ds.Tables["Users"].Rows[0][7].ToString());
				if(ds.Tables["Users"].Rows[0][8].ToString()== "True")
					locked = true;
				else
					locked = false;
				User u = new User(name, pass, key, share, access, avail, locked, uid);
				return u;
			}
			else
				return null;
		}

		/// <summary>
		/// Adds the users IP-address to the database
		/// </summary>
		/// <param name="uName">the user to change</param>
		/// <param name="address">the "new" ip-address</param>
		public void SaveIP(string uName, string address)
		{
			string query = "UPDATE Users SET ip = '" + address + "' WHERE Users.userName = '" + uName + "'";
			DataSet ds = new DataSet();
			OleDbConnection dbConnection = new OleDbConnection(connStr);
			OleDbCommand dbCommand = new OleDbCommand(query);

			try
			{
				dbCommand.Connection = dbConnection;
				dbConnection.Open();
				dbCommand.ExecuteNonQuery();
			}
			catch(Exception ex)
			{
				MessageBox.Show(ex.Message, "Hub.DBHandle.SaveIP");
			}
			finally
			{
				dbCommand.Connection.Close();
			}
		}

		/// <summary>
		/// Saves the user's share
		/// </summary>
		/// <param name="uN">username</param>
		/// <param name="s">size of the share</param>
		/// <param name="av">new available size</param>
		/// <param name="ac">new accessible space</param>
		public void SaveShare(string uN, long s, long av, long ac)
		{
			string query = "UPDATE Users SET share = " + s + ", available = " + av + ", accessible = " + ac + " WHERE Users.userName = '" + uN + "'";
			DataSet ds = new DataSet();
			OleDbConnection dbConnection = new OleDbConnection(connStr);
			OleDbCommand dbCommand = new OleDbCommand(query);

			try
			{
				dbCommand.Connection = dbConnection;
				dbConnection.Open();
				dbCommand.ExecuteNonQuery();
			}
			catch(Exception ex)
			{
				MessageBox.Show(ex.Message, "Hub.DBHandle.SaveShare");
			}
			finally
			{
				dbCommand.Connection.Close();
			}
		}

		/// <summary>
		/// Saves the user's public key
		/// </summary>
		/// <param name="uN">username</param>
		/// <param name="key">public key</param>
		public void SaveKey(string uN, string key)
		{
			string query = "UPDATE Users SET pubkey = '" + key + "' WHERE Users.userName = '" + uN + "'";
			DataSet ds = new DataSet();
			OleDbConnection dbConnection = new OleDbConnection(connStr);
			OleDbCommand dbCommand = new OleDbCommand(query);

			try
			{
				dbCommand.Connection = dbConnection;
				dbConnection.Open();
				dbCommand.ExecuteNonQuery();
			}
			catch(Exception ex)
			{
				MessageBox.Show(ex.Message, "Hub.DBHandle.SaveKey");
			}
			finally
			{
				dbCommand.Connection.Close();
			}
		}

		/// <summary>
		/// Updates the user's accessible space
		/// </summary>
		/// <param name="uN">username</param>
		/// <param name="acc">new accessible space, space left to use for backup</param>
		public void ChangeAccessible(string uN, long acc)
		{
			string query = "UPDATE Users SET accessible = " + acc + " WHERE Users.userName = '" + uN + "'";
			DataSet ds = new DataSet();
			OleDbConnection dbConnection = new OleDbConnection(connStr);
			OleDbCommand dbCommand = new OleDbCommand(query);

			try
			{
				dbCommand.Connection = dbConnection;
				dbConnection.Open();
				dbCommand.ExecuteNonQuery();
			}
			catch(Exception ex)
			{
				MessageBox.Show(ex.Message, "Hub.DBHandle.ChangeAccessible");
			}
			finally
			{
				dbCommand.Connection.Close();
			}
		}

		/// <summary>
		/// Updates the user's available space
		/// </summary>
		/// <param name="uN">username</param>
		/// <param name="avail">new available space, space left in the user's own share</param>
		public void ChangeAvailable(string uN, long avail)
		{
			string query = "UPDATE Users SET available = " + avail + " WHERE Users.userName = '" + uN + "'";
			DataSet ds = new DataSet();
			OleDbConnection dbConnection = new OleDbConnection(connStr);
			OleDbCommand dbCommand = new OleDbCommand(query);

			try
			{
				dbCommand.Connection = dbConnection;
				dbConnection.Open();
				dbCommand.ExecuteNonQuery();
			}
			catch(Exception ex)
			{
				MessageBox.Show(ex.Message, "Hub.DBHandle.ChangeAvailable");
			}
			finally
			{
				dbCommand.Connection.Close();
			}
		}
		
		/// <summary>
		/// Adds a file that is ready to be back upped
		/// </summary>
		/// <param name="id">the owner's userid</param>
		/// <param name="fName">filename</param>
		/// <param name="fSize">file size</param>
		/// <param name="dest">destination</param>
		public void AddFile(int id, string fName, long fSize, int dest)
		{
			string query = "INSERT INTO Files (uid, filename, fsize, destination) VALUES (" + id + ", '" + fName + "', " + fSize + ", " + dest + ")";
			OleDbConnection dbConnection = new OleDbConnection(connStr);
			OleDbCommand dbCommand = new OleDbCommand(query);

			try
			{
				dbCommand.Connection = dbConnection;
				dbConnection.Open();
				dbCommand.ExecuteNonQuery();
			}	
			catch(Exception ex)
			{
				MessageBox.Show(ex.Message, "Hub.DBHandle.AddFile");
			}
			finally
			{
				dbCommand.Connection.Close();
			}
		}

		/// <summary>
		/// marks a file as back upped and transferred
		/// </summary>
		/// <param name="id">the owner's userid</param>
		/// <param name="fName">filename</param>
		/// <param name="destName">destination's username</param>
		public void FileDone (int id, string fName, string destName)
		{
			int destId = GetUserId(destName);

			string query = "UPDATE Files SET done = true WHERE Files.uid = " + id + " AND Files.filename = '" + fName + "' AND Files.destination = " + destId + "";
			DataSet ds = new DataSet();
			OleDbConnection dbConnection = new OleDbConnection(connStr);
			OleDbCommand dbCommand = new OleDbCommand(query);

			try
			{
				dbCommand.Connection = dbConnection;
				dbConnection.Open();
				dbCommand.ExecuteNonQuery();
			}
			catch(Exception ex)
			{
				MessageBox.Show(ex.Message, "Hub.DBHandle.FileDone");
			}
			finally
			{
				dbCommand.Connection.Close();
			}
		}

		/// <summary>
		/// Gets the userid for this client
		/// </summary>
		/// <param name="destName">the user's username</param>
		/// <returns>the user's id</returns>
		private int GetUserId (string destName)
		{
			string query = "SELECT Users.uid FROM Users WHERE Users.username = '" + destName + "'";

			DataSet ds = new DataSet();
			OleDbConnection dbConnection = new OleDbConnection(connStr);

			try
			{
				OleDbDataAdapter dataAdapter = new OleDbDataAdapter(query, connStr);
				dataAdapter.Fill (ds, "Users");
			}
			catch(Exception ex)
			{
				MessageBox.Show(ex.Message, "Hub.DBHandle.GetUserId");
			}
			finally
			{
				dbConnection.Close ();
			}

			// check if there's anything in the dataset
			int count = ds.Tables["Users"].Rows.Count;
			if (count > 0)
			{
				int uid = Int32.Parse(ds.Tables["Users"].Rows[0][0].ToString());			
				return uid;
			}
			else
				return 0;
		}

		/// <summary>
		/// Deletes a file
		/// </summary>
		/// <param name="uid">the owner's user id</param>
		/// <param name="filename">filename</param>
		/// <param name="duid">destination's user id</param>
		public void DeleteFile(int uid, string filename, int duid)
		{
			string query = "DELETE FROM Files WHERE uid = " + uid + " AND filename = '" + filename + "' AND destination = " + duid + "" ;
			OleDbConnection dbConnection = new OleDbConnection(connStr);
			OleDbCommand dbCommand = new OleDbCommand(query);

			try
			{
				dbCommand.Connection = dbConnection;
				dbConnection.Open();
				dbCommand.ExecuteNonQuery();
			}	
			catch(Exception ex)
			{
				MessageBox.Show(ex.Message, "Hub.DBHandler.DeleteFile");
			}
			finally
			{
				dbCommand.Connection.Close();
			}	
		}
	}
}
