using System;
using System.Collections;
using System.Windows.Forms;

namespace P2PBackupHub
{
	/// <summary>
	/// Holds all online users
	/// </summary>
	public class UserList:ArrayList
	{
		public UserList()
		{
	
		}

		/// <summary>
		/// Adds a verified user to the UserList
		/// </summary>
		/// <param name="user">the user to add</param>
		public void AddUser (User user)
		{
			user.RemoveUserEvent += new P2PBackupHub.User.RemoveUserHandler(RemoveUser);
			user.CalculateDestinationEvent += new P2PBackupHub.User.CalculateDestinationHandler(CalculateDestination);
			user.GetClientEvent += new P2PBackupHub.User.GetClientHandler(GetClient);
			this.Add (user);	
		}

		/// <summary>
		/// Removes a user from the UserList, the user is no longer online
		/// </summary>
		/// <param name="user">the user to remove</param>
		public void RemoveUser (User user)
		{
			this.Remove (user);
		}

		/// <summary>
		/// Randomly chooses a destination among online users with enough space
		/// </summary>
		/// <param name="u">the user who will get a destination</param>
		/// <returns>the destination user</returns>
		public User CalculateDestination(User u)
		{
			User destination = new User();
			Random rand = new Random((int)DateTime.Now.Ticks);
			int no;
			int count = 0;

			// if only 1 client is online
			if (this.Count == 1)
				return null;

			// check if any client other than me has enough space
			foreach (User user in this)
			{
				if (user.Available > u.Filesize)
					if (user.UserName != u.UserName)
						count++;
			}

			// if some client has enough space
			if (count > 0)
			{
				do
				{
					no = rand.Next(this.Count);
					destination = (User)this[no];
				}
				while (u.UserName == destination.UserName || destination.Available < u.Filesize);
				
				return destination;
			}
			else 
				return null;
		}

		/// <summary>
		/// Gets a user
		/// </summary>
		/// <param name="uName">the sought user's name</param>
		/// <returns>the sought user or null if not found</returns>
		public User GetClient(string uName)
		{
			foreach (User u in this)
			{
				if (u.UserName == uName)
					return u;
			}
			return null;	
		}
	}
}
